local BLOODTRADER_NPCID = 115264
local BLOOD_ITEMID = 124124
local BLOODTRADER_CACHE_INDEX = 1
local RESOURCES_CURRENCYID = 1220

-- helper function to check if we are interacting with the blood trader npc
local function IsBloodTrader()
  local guid = UnitGUID("npc")
  if not guid then return end
  local _, _, _, _, _, npcid = strsplit("-", guid)
  if not npcid then return end
  if tonumber(npcid) ~= BLOODTRADER_NPCID then return end
  return true
end



-- Hook GetMerchantItemInfo but only when it's called from within
-- FrameXML/MerchantFrame.lua's MerchantItemButton_OnModifiedClick() function
-- This makes the UI think that the Blood Trader only sells stacks of 1 for
-- the purposes of allowing split stacks.
local oldGetMerchantItemInfo = GetMerchantItemInfo
GetMerchantItemInfo = function(index, ...)
  -- ignore if not blood trader
  if not IsBloodTrader() then return oldGetMerchantItemInfo(index, ...) end

  -- get original values
  local name, texture, price, quantity, numAvailable, isUsable, extendedCost = oldGetMerchantItemInfo(index)
  
  -- only change quantity if GetMerchantItemInfo is being called from MerchantItemButton_OnModifiedClick
  -- checking for the filename+linenumber instead of function name allows this to still work if the function
  -- is called from another name (like if an addon hooks it and calls from a local variable, like we do)
  local stack = debugstack(2,1,0)
  if stack:match("Interface\\FrameXML\\MerchantFrame.lua:489") then quantity = 1 end
  
  -- return values with quantity possibly modified
  return name, texture, price, quantity, numAvailable, isUsable, extendedCost  
end
-- Do the same thing but with maxStack for the resource cache
local oldGetMerchantItemMaxStack = GetMerchantItemMaxStack
GetMerchantItemMaxStack = function(index, ...)
  -- ignore if not blood trader
  if not IsBloodTrader() or index ~= BLOODTRADER_CACHE_INDEX then return oldGetMerchantItemMaxStack(index, ...) end

  -- get original values
  local maxStack = oldGetMerchantItemMaxStack(index)

  -- only change quantity if GetMerchantItemMaxStack is being called from MerchantItemButton_OnModifiedClick
  local stack = debugstack(2,1,0)
  if stack:match("Interface\\FrameXML\\MerchantFrame.lua:488") then maxStack = MainMenuBarBackpackButton.freeSlots end
  
  -- return values with quantity possibly modified
  return maxStack
end



-- Show blood of sargeras as a currency in lower right of vendor window
MerchantFrame:HookScript("OnEvent", function(self,event,...)
  if event == "MERCHANT_CLOSED" then
    self:UnregisterEvent("BAG_UPDATE")
  elseif event == "MERCHANT_SHOW" and IsBloodTrader() then
    self:RegisterEvent("BAG_UPDATE")
  elseif event == "BAG_UPDATE" then
    MerchantFrame_OnEvent(self, "CURRENCY_DISPLAY_UPDATE")
  end
end)
hooksecurefunc("MerchantFrame_Update", MerchantFrame_UpdateCurrencies)
-- if blood trader, return blood itemid and order resources as currencies to display
local oldGetMerchantCurrencies = GetMerchantCurrencies
GetMerchantCurrencies = function(...)
  if IsBloodTrader() then
    return -BLOOD_ITEMID, RESOURCES_CURRENCYID
  else
    return oldGetMerchantCurrencies(...)
  end
end
-- treat negative currencyids as an item id
local oldGetCurrencyInfo = GetCurrencyInfo
GetCurrencyInfo = function(id, ...)
  if id >= 0 then return oldGetCurrencyInfo(id, ...) end
  
  -- convert negative currencyid to itemid
  id = -id
  
  local name, _, quality = GetItemInfo(id)
  local texture = GetItemIcon(id)

  --local name, amount, texturePath, earnedThisWeek, weeklyMax, totalMax, isDiscovered, quality = GetCurrencyInfo(id)
  return name or "", GetItemCount(id, true), texture, 0, 0, 0, true, quality or 1
end
-- handle negative currencyid tooltips
hooksecurefunc(GameTooltip, "SetCurrencyByID", function(self, id)
  if id >= 0 then return end
  self:SetItemByID(-id)
end)



-- allow people to buy multiple caches AND removes confirmation when buying stacks
hooksecurefunc("MerchantFrame_Update", function()
  -- ignore if not blood trader
  if not IsBloodTrader() then return end
  
  -- replace splitstack functions if not already done so
  for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
    local itemButton = _G["MerchantItem"..i.."ItemButton"];
    if not itemButton.oldSplitStack then
      itemButton.oldSplitStack = itemButton.SplitStack
      
      itemButton.SplitStack = function(button, split, ...)
        if not IsBloodTrader() then return button.oldSplitStack(button, split, ...) end

        local _, _, _, stackCount = oldGetMerchantItemInfo(button:GetID())

        -- round up to multiple of stack size
        if split % stackCount ~= 0 then
          split = split + stackCount
          local overage = split % stackCount
          split = split - overage
        end
  
        -- if this is first item (resource cache), buy 1 multiple times
        if button:GetID() == BLOODTRADER_CACHE_INDEX then
          split = min(split, MainMenuBarBackpackButton.freeSlots)
          while split > 0 do
            BuyMerchantItem(button:GetID(), 1)
            BuyMerchantItem(0, 0) -- slight delay to prevent "item is busy" errors
            BuyMerchantItem(0, 0)
            split = split - 1
          end
        -- otherwise just buy amount
        else
          BuyMerchantItem(button:GetID(), split)
        end
      end --end new SplitStack function
      
    end
  end  
end)
